﻿//================================================================
//	equation.js
//================================================================

//------------------------------------------------------
//	連立方程式を解く
//		①入力データをdataMatrixに変換
//		②逆行列を求める
//		③解を求める
//------------------------------------------------------
function mxcalc(inputData) {

	var data, nc, rank, i, j, r;
	var r = new Array();
	var cons = new Array();
	var temp;
	
	data = getdata(inputData);

	if (data == false) {
		return "err01";					//入力データが不正

	} else {
		nc = data.length;
		rank = data[0].length-1;
		
		if (nc != rank) {
			return "err01";				//入力は、係数行列＋定数ベクトルが必須
		
		} else {
			for (var i = 0; i < nc; i++) {
				r[i] = new Array(rank);
				for (var j = 0; j < rank; j++) {
					r[i][j] = data[i][j];
				}
				cons[i] = data[i][j];
			}

			if (inv(r, rank, 1e-10) == 1) {
				return "err02";			//逆行列が求められない
			
			} else {
				var tmp = "";
				for (var i = 0; i < rank; i++) {
					temp = 0;
					for (var j = 0; j < rank; j++) {
						//******************************************BigNumber
						//temp += r[i][j] * cons[j];
						var _t = BigNumber(r[i][j]).times(cons[j]);
						temp = BigNumber(temp).plus(_t);
					}
					
					if (tmp == "")
						tmp = temp;
					else
						tmp = tmp + "," + temp;
				}
				
				return tmp;
			}
		}
	}
}

//----------------------------------------------------
//	入力データをdataMatrixに変換
//----------------------------------------------------
function getdata(data) {
	var lines, vars, nv, nc, j0, nv0, i, j;
	var dataMatrix = new Array();

	lines = data.split("\n");
	
	nc = 0
	for (i = 0; i < lines.length; i++) {
		if (lines[i] != "") {
			vars = lines[i].split(",");
			nv = vars.length;
			dataMatrix[nc] = new Array();
			j0 = 0;
			for (j = 0; j < nv; j++) {
				if (vars[j] == "") {
					continue;
				}
				dataMatrix[nc][j0] = parseFloat(vars[j]);
				if (isNaN(dataMatrix[nc][j0++])) {
					return false;
				}
			}
			nc++;
		}
	}
	if (nc == 0) {
		return false;
	}
	
	return dataMatrix;
}

//------------------------------------------------------
//	逆行列を求める
//------------------------------------------------------
function inv(r, ip, epsinv) {
	var i, j, k, b, t, m, l;
	m = new Array(r);
	l = new Array(r);
	for (k = 0; k < ip; k++) {
		m[k] = l[k] = k; 
		b = r[k][k];
		for (var j = k; j < ip; j++) {
			for (var i = k; i < ip; i++) {
				if (Math.abs(r[j][i]) > Math.abs(b)) {
					b = r[j][i];
					l[k] = i;
					m[k] = j;
				}
			}
		}
		if (Math.abs(b) < epsinv) {
			return 1;
		}
		j = l[k];
		if (j > k) {
			for (var i = 0; i < ip; i++) {
				//******************************************BigNumber
				//t = -r[i][k];
				t = BigNumber(r[i][k]).times(-1);
				r[i][k] = r[i][j];
				r[i][j] = t;
			}
		}
		i = m[k];
		if (i > k) {
			for (var j = 0; j < ip; j++) {
				//******************************************BigNumber
				//t = -r[k][j];
				t = BigNumber(r[k][j]).times(-1);
				r[k][j] = r[i][j];
				r[i][j] = t;
			}
		}
		for (var i = 0; i < ip; i++) {
			if (i != k) {
				//******************************************BigNumber
				//r[k][i] /= -b;
				r[k][i] = BigNumber(r[k][i]).div(b);
			}
		}
		for (var i = 0; i < ip; i++) {
			for (var j = 0; j < ip; j++) {
				if (i != k && j != k) {
					//******************************************BigNumber
					//r[j][i] += r[k][i]*r[j][k];
					var _t = BigNumber(r[k][i]).times(r[j][k]);
					r[j][i] = BigNumber(r[j][i]).plus(_t);
				}
			}
		}
		for (var j = 0; j < ip; j++) {
			if (j != k) {
				//******************************************BigNumber
				//r[j][k] /= b;
				r[j][k] = BigNumber(r[j][k]).div(b);
			}
		}
		//******************************************BigNumber
		//r[k][k] = 1.0/b;
		r[k][k] = BigNumber(1.0).div(b);
   }
   for (var k = ip-2; k >= 0; k--) {
		i = l[k];
		if (i > k) {
			for (var j = 0; j < ip; j++) {
				t = r[k][j];
				//******************************************BigNumber
				//r[k][j] = -r[i][j];
				r[k][j] = BigNumber(r[i][j]).times(-1);
				r[i][j] = t;
			}
		}
		j = m[k];
		if (j > k) {
			for (var i = 0; i < ip; i++) {
				t = r[i][k];
				//******************************************BigNumber
				//r[i][k] = -r[i][j];
				r[i][k] = BigNumber(r[i][j]).times(-1);
				r[i][j] = t;
			}
		}
	}
	return 0;
}
